function [hell] = make_ellipsoid_from_vertices(V, c)
%MAKE_ELLIPSOID_FROM_VERTICES Constructs a new d-dimensional ellipsoid 
%                 with the given "positive vertices", and size parameter, 
%                 c.  The constructed ellipsoid is centered at the origin.
%                 Note that the input vertices (i.e., the columns of V)
%                 must therefore be orthogonal vectors, themselves 
%                 centered at the origin.
%
%                 The size parameter, c, may be needed because the 
%                 points alone only determine the eigenvalues up to 
%                 a positive constant.  For vertices which fall on the 
%                 constructed ellipsoid, choose as the size parameter
%                 c = 1.
% 
%                 The new ellipsoid is centered at the origin.
%
% Syntax:      [hell] = make_ellipsoid_from_vertices(V, c)
%
% Inputs:
%              V - a d x d array of positive vertices (in columns)
%              c - the size parameter of the new ellipsoid
%
% Outputs:
%              hell - a new ellipsoid, centered at the origin, 
%                 with the given vertices
%
% Example: 
%              V = [.56569, .21213; -.56569, .21213;];
%              e = make_ellipsoid_from_vertices(V, 1);
%              mu = get(e, 'center');
%              assertEquals('Center is 0,0', mu(1, 1), 0);
%              assertEquals('Center is 0,0', mu(2, 1), 0);
%              V2 = vertices(e);
%              assertEquals('Ratios ok', V(1,1)/V(2,1), V2(1,1)/V2(2,1));
%              assertEquals('Ratios ok', V(1,2)/V(2,2), V2(1,2)/V2(2,2));
%
% Other m-files required: none
% Subfunctions: none
% MAT-files required: none
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EITHER EXPRESS, IMPLIED, STATUTORY OR OTHERWISE INCLUDING BUT NOT LIMITED TO
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, FREE OF
% DEFECTS AND NONINFRINGEMENT. 
% 
% UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY OF LIABILITY, WHETHER TORT
% (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE SHALL THE AUTHORS OR THEIR 
% EMPLOYERS BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR 
% CONSEQUENTIAL DAMAGES OF ANY KIND, INCLUDING, WITHOUT LIMITATION, DAMAGES 
% FOR LOST PROFITS, LOSS OF GOODWILL, WORK STOPPPAGE, COMPUTER FAILURE OR 
% MALFUNCTION, LOSS OF USE, LOSS OF DATA, OR ANY AND ALL OTHER COMMERCIAL 
% DAMAGES AND LOSSES WHETHER OR NOT ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, 
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
% THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH 
% YOU. IF THE SOFTWARE PROVES DEFECTIVE IN ANY RESPECT, YOU ASSUME THE COST OF 
% ANY NECESSARY SERVICING, REPAIR OR CORRECTION  
% THIS SOFTWARE IS RELEASED INTO THE PUBLIC DOMAIN, OCT 2012.

% Author: Mark D. Flood
% Research and Analysis Center, Office of Financial Research
% Oct 2012; Last revision: 21-Nov-2010

%------------- BEGIN CODE --------------
    d = length(V);
    L = zeros(d);
    S = zeros(d);
    lambda = zeros(d);
    for i=1:d;
%         disp(['  - vertex ', sprintf('%.0f', i), ': ']);
        % The length of each vertex vector
        L(i,i) = norm(V(:,i));
        % S is a set of orthonormal eigenvectors
        S(:,i) = V(:,i)/L(i,i);
        % lambda is the matrix of eigenvalues (note the scaling by c)
        lambda(i,i) = c/(L(i,i)*L(i,i));
        % Test that the vectors are all orthogonal:
        for j=i+1:d;
            Vi = V(:,i);
            Vj = V(:,j);
            cosine = Vi'*Vj/(norm(Vi)*norm(Vj));
            if (abs(cosine)>.000001) 
                error(['Vertices not orthogonal: ' num2str(i) ', ' num2str(j)]);
            end
        end;
    end;
    hell = hyperellipsoid(zeros(d, 1), S*lambda*S', c);
end
