function [hell, tfm] = rotate_to_coordaxes(hellipse)
%ROTATE_TO_COORDAXES Rotates the ellipsoid (hellipse) so its principal
%                 axes align with the coordinate axes.  Both ellipsoids
%                 are centered at the origin.  Note that there are
%                 (2^d)*d! valid ways to rotate the ellipsoid to the axes.
%                 This algorithm does not prescribe which solution will
%                 be provided.
%
% Syntax:      [hell, tfm] = rotate_to_coordaxes(hellipse)
%
% Inputs:
%              hellipse - the shape to be rotated, must be centered at the
%                 origin
%
% Outputs:
%              hell - hell, a new hyperellipsoid, rotated to the 
%                 coordinate axes
%              tfm - the transformation matrix that creates the rotation
%
% Example: 
%              mu = zeros(5, 1);
%              A = [ 0.56569, 0.00000, 0.00000, 0.00000, 0.00000; 
%                    0.56569, 0.21213, 0.00000, 0.00000, 0.00000; 
%                   -0.56569, 0.21213, 0.44567, 0.00000, 0.00000; 
%                    0.56569, 0.41213, 0.70000,-0.44567, 0.00000; 
%                    0.56569, 0.21213, 1.23455, 0.33001, 2.45677;];
%              sig = inv(A*A');
%              d = length(mu);
%              e1 = hyperellipsoid(mu, sig, 5);
%              [e2, tfm] = rotate_to_coordaxes(e1);
%              v1 = vertices(e1);
%              v2 = vertices(e2);
%              for i=1:d;
%                  for j=1:d;
%                      if (i==j)
%                         assertTrue('Diags ok: ', abs(v2(i, j))~=0);
%                      else
%                         assertTrue('Off-diags ok: ', abs(v2(i, j))<EPS);
%                      end
%                  end
%              end;
%
% Other m-files required: none
% Subfunctions: none
% MAT-files required: none
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EITHER EXPRESS, IMPLIED, STATUTORY OR OTHERWISE INCLUDING BUT NOT LIMITED TO
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, FREE OF
% DEFECTS AND NONINFRINGEMENT. 
% 
% UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY OF LIABILITY, WHETHER TORT
% (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE SHALL THE AUTHORS OR THEIR 
% EMPLOYERS BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR 
% CONSEQUENTIAL DAMAGES OF ANY KIND, INCLUDING, WITHOUT LIMITATION, DAMAGES 
% FOR LOST PROFITS, LOSS OF GOODWILL, WORK STOPPPAGE, COMPUTER FAILURE OR 
% MALFUNCTION, LOSS OF USE, LOSS OF DATA, OR ANY AND ALL OTHER COMMERCIAL 
% DAMAGES AND LOSSES WHETHER OR NOT ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, 
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
% THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH 
% YOU. IF THE SOFTWARE PROVES DEFECTIVE IN ANY RESPECT, YOU ASSUME THE COST OF 
% ANY NECESSARY SERVICING, REPAIR OR CORRECTION  
% THIS SOFTWARE IS RELEASED INTO THE PUBLIC DOMAIN, OCT 2012.

% Author: Mark D. Flood
% Research and Analysis Center, Office of Financial Research
% Oct 2012; Last revision: 21-Nov-2010

%------------- BEGIN CODE --------------
    V = vertices(hellipse);
    d = length(V);
    V2 = zeros(d,d);
    for i=1:d;
        V2(i,i) = norm(V(:,i));
    end;
    tfm = V2/V;
    hell = transform_ellipsoid(hellipse, tfm);
end
