function [dx2d] = univariate_shocks(hellipse)
%UNIVARIATE_SHOCKS Calculates 2d d-dimensional univariate shocks (up and
%                 down in each of the d dimensions) based on the given 
%                 ellipsoid.  Univariate shocks are points on the surface
%                 of the ellipsoid that differ from the center of the 
%                 ellipsoid in only one dimension.  Thus, for an ellipsoid
%                 centered at the origin, only one element of a
%                 d-dimensional shock will be non-zero.  This function
%                 does not assume that the ellipsoid is centered at
%                 the origin.
%
% Syntax:      [dx2d] = univariate_shocks(hellipse)
%
% Inputs:
%              hellipse - the basis for the shocks; it must have 
%                 measurable width in every dimension
%
% Outputs:
%              dx2d - d x 2d array, [dx2d], with each column a shock;
%                 the first d columns are positive univariate shocks, and
%                 final d columns are matching negative univariate shocks
%
% Example: 
%              mu = [9; 6];
%              sig = 3*eye(2);
%              hellp = hyperellipsoid(mu, sig, 5);
%              shocks = univariate_shocks(hellp);
%
% Other m-files required: none
% Subfunctions: none
% MAT-files required: none
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EITHER EXPRESS, IMPLIED, STATUTORY OR OTHERWISE INCLUDING BUT NOT LIMITED TO
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, FREE OF
% DEFECTS AND NONINFRINGEMENT. 
% 
% UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY OF LIABILITY, WHETHER TORT
% (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE SHALL THE AUTHORS OR THEIR 
% EMPLOYERS BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR 
% CONSEQUENTIAL DAMAGES OF ANY KIND, INCLUDING, WITHOUT LIMITATION, DAMAGES 
% FOR LOST PROFITS, LOSS OF GOODWILL, WORK STOPPPAGE, COMPUTER FAILURE OR 
% MALFUNCTION, LOSS OF USE, LOSS OF DATA, OR ANY AND ALL OTHER COMMERCIAL 
% DAMAGES AND LOSSES WHETHER OR NOT ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, 
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
% THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH 
% YOU. IF THE SOFTWARE PROVES DEFECTIVE IN ANY RESPECT, YOU ASSUME THE COST OF 
% ANY NECESSARY SERVICING, REPAIR OR CORRECTION  
% THIS SOFTWARE IS RELEASED INTO THE PUBLIC DOMAIN, OCT 2012.

% Author: Mark D. Flood
% Research and Analysis Center, Office of Financial Research
% Oct 2012; Last revision: 21-Nov-2010

%------------- BEGIN CODE --------------
    [ellipsoid2, mu] = center_at_origin(hellipse);
    sig = hellipse.shape;
    c = hellipse.size;
    d = length(mu);
    dx2d = zeros(d,2*d);
    for i=1:d;
        dx2d(i,i) = sqrt(c/sig(i,i));
        dx2d(i,2*i) = -sqrt(c/sig(i,i));
    end;
    for i=1:2*d;
        dx2d(:,i) = dx2d(:,i) + mu;
    end;
end
